package net.corda.attestation.host.sgx.entities

import net.corda.attestation.host.sgx.enclave.ECKey

/**
 * Signed report from the application enclave. The corresponds to message 3 in
 * the Intel remote attestation flow.
 */
class Quote(
    /**
     * The 128-bit AES-CMAC generated by the application enclave. See
     * [sgx_ra_msg3_t](https://software.intel.com/en-us/node/709238) for
     * more details on its derivation.
     */
    val messageAuthenticationCode: ByteArray,

    /**
     * The public elliptic curve key of the application enclave.
     */
    val publicKey: ECKey,

    /**
     * Security property of the Intel SGX Platform Service. If the Intel
     * SGX Platform Service security property information is not required
     * in the remote attestation and key exchange process, this field will
     * be all zeros. The buffer is 256 bytes long.
     */
    val securityProperties: ByteArray,

    /**
     * Quote returned from sgx_get_quote. More details about how the quote
     * is derived can be found in Intel's documentation:
     * [sgx_ra_msg3_t](https://software.intel.com/en-us/node/709238)
     */
    val payload: ByteArray
)
